package com.luojigou.hardware.xiaodou

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.luojigou.hardware.xiaodou.connect.XDConnectConnectingFragment
import com.luojigou.hardware.xiaodou.connect.XDConnectErrorFragment
import com.luojigou.hardware.xiaodou.connect.XDConnectFailFragment
import com.luojigou.hardware.xiaodou.connect.XDConnectNotFoundFragment
import com.luojigou.hardware.xiaodou.connect.XDConnectPasswordErrorFragment
import com.luojigou.hardware.xiaodou.connect.XDConnectSuccessFragment
import com.luojigou.hardware.xiaodou.connect.XDConnectTimeoutFragment
import com.luojigou.hardware.xiaodou.wifi.XDScanWifiInfo
import com.luojigou.xiaodou.R

class XDConnectActivity : AppCompatActivity() {
    companion object {
        fun startConnect(context: Context, device: XDScanWifiInfo) {
            val intent = Intent(context, XDConnectActivity::class.java)
            intent.putExtra(XDConnectService.ACTION_SELECT_WIFI_DEVICE_DATA, device)
            context.startActivity(intent)
        }
    }

    private val timeoutFragment = XDConnectTimeoutFragment()
    private val errorFragment = XDConnectErrorFragment()
    private val successFragment = XDConnectSuccessFragment()
    private val connectingFragment = XDConnectConnectingFragment()
    private val failFragment = XDConnectFailFragment()
    private val notFoundFragment = XDConnectNotFoundFragment()
    private val passwordErrorFragment = XDConnectPasswordErrorFragment()

    private val connectReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent) {
            setFragment(
                when (intent.action) {
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_TIMEOUT -> timeoutFragment
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_ERROR -> errorFragment
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_SUCCESS -> successFragment
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_CONNECTING -> connectingFragment
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_FAIL -> failFragment
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_NOT_FOUND -> notFoundFragment
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_PASSWORD_ERROR -> passwordErrorFragment
                    else -> errorFragment
                }
            )
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_connect)
        setFragment(connectingFragment)

        val intentFilter = IntentFilter()
        intentFilter.apply {
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_TIMEOUT)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_ERROR)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_SUCCESS)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_CONNECTING)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_FAIL)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_NOT_FOUND)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_PASSWORD_ERROR)
        }

        ContextCompat.registerReceiver(
            this,
            connectReceiver,
            intentFilter,
            ContextCompat.RECEIVER_EXPORTED,
        )

        val device = intent.getParcelableExtra<XDScanWifiInfo>(XDConnectService.ACTION_SELECT_WIFI_DEVICE_DATA)

        if (device != null) {
            val intent = Intent(XDConnectService.ACTION_SELECT_WIFI_DEVICE)
            intent.putExtra(XDConnectService.ACTION_SELECT_WIFI_DEVICE_DATA, device)
            sendBroadcast(intent)
        }
    }

    private fun setFragment(fragment: Fragment) {
        val fragmentManager = supportFragmentManager;
        val beginTransaction = fragmentManager.beginTransaction()
        if (fragmentManager.findFragmentById(R.id.fragment_container) == null) {
            beginTransaction.add(R.id.fragment_container, fragment)
        } else {
            beginTransaction.replace(R.id.fragment_container, fragment)
        }
        beginTransaction.commit()
    }

    override fun onDestroy() {
        unregisterReceiver(connectReceiver)
        super.onDestroy()
    }
}