package com.luojigou.hardware.xiaodou

import android.Manifest
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothManager
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.fragment.app.Fragment
import com.luojigou.hardware.xiaodou.ble.XDScanBLEDisableFragment
import com.luojigou.hardware.xiaodou.ble.XDScanBLENotSupportFragment
import com.luojigou.hardware.xiaodou.ble.XDScanBLEPermissionFragment
import com.luojigou.hardware.xiaodou.ble.XDScanBLEStatus
import com.luojigou.xiaodou.R


class XDScanBLEActivity : AppCompatActivity(), XDScanBLEStatus.Host {
    private val permissions: Array<String> by lazy {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
            arrayOf(
                Manifest.permission.BLUETOOTH_SCAN,
                Manifest.permission.BLUETOOTH_ADVERTISE,
                Manifest.permission.BLUETOOTH_CONNECT
            )
        } else {
            arrayOf(
                Manifest.permission.ACCESS_FINE_LOCATION
            )
        }
    }

    private val disableFragment = XDScanBLEDisableFragment(this)
    private val normalFragment = com.luojigou.hardware.xiaodou.ble.XDScanBLENormalFragment(this)
    private val permissionFragment = XDScanBLEPermissionFragment(this)
    private val notsupportFragment = XDScanBLENotSupportFragment(this)


    private val bluetoothManager: BluetoothManager by lazy {
        getSystemService(BLUETOOTH_SERVICE) as BluetoothManager
    }

    private val broadcastReceiver by lazy {
        object : BroadcastReceiver() {
            override fun onReceive(context: Context?, intent: Intent?) {
                val lastState = intent?.getIntExtra(BluetoothAdapter.EXTRA_PREVIOUS_STATE, -1)
                val currentState = intent?.getIntExtra(BluetoothAdapter.EXTRA_STATE, -1)

                if (lastState == currentState) {
                    return
                }

                when (currentState) {
                    BluetoothAdapter.STATE_OFF -> {
                        checkBluetoothPermissionAndEnable()
                    }

                    BluetoothAdapter.STATE_ON -> {
                        checkBluetoothPermissionAndEnable()
                    }
                }
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_scanble)
    }

    override fun onResume() {
        super.onResume()
        registerReceiver(broadcastReceiver, IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED))
        checkBluetoothPermissionAndEnable()
    }

    override fun onPause() {
        unregisterReceiver(broadcastReceiver)
        super.onPause()
    }

    private fun checkBluetoothPermissionAndEnable() {
        val bluetoothAvailable = packageManager.hasSystemFeature(PackageManager.FEATURE_BLUETOOTH)
        val bluetoothLEAvailable =
            packageManager.hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE)

        if (!bluetoothAvailable || !bluetoothLEAvailable) {
            setFragment(notsupportFragment)
            return
        }

        if (permissions.find {
                ActivityCompat.checkSelfPermission(
                    this, it
                ) != PackageManager.PERMISSION_GRANTED
            } != null) {
            setFragment(permissionFragment)
            return
        }

        if (bluetoothManager.adapter == null || !bluetoothManager.adapter.isEnabled) {
            setFragment(disableFragment)
            return
        }

        setFragment(normalFragment)
    }

    private fun setFragment(fragment: Fragment) {
        val fragmentManager = supportFragmentManager;
        val beginTransaction = fragmentManager.beginTransaction()
        if (fragmentManager.findFragmentById(R.id.fragment_container) == null) {
            beginTransaction.add(R.id.fragment_container, fragment)
        } else {
            beginTransaction.replace(R.id.fragment_container, fragment)
        }
        beginTransaction.commit()
    }

    override fun onStatusChanged(status: XDScanBLEStatus) =
        checkBluetoothPermissionAndEnable()
}
