package com.luojigou.hardware.xiaodou.ble

import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.luojigou.xiaodou.R
import com.luojigou.hardware.xiaodou.XDConnectService
import com.luojigou.hardware.xiaodou.XDConnectService.Companion.ACTION_SCAN_RESULT
import com.luojigou.hardware.xiaodou.XDConnectService.Companion.ACTION_SCAN_RESULT_DATA
import com.luojigou.hardware.xiaodou.XDConnectService.Companion.ACTION_START_SCAN
import com.luojigou.hardware.xiaodou.XDConnectService.Companion.ACTION_STOP_SCAN
import com.luojigou.hardware.xiaodou.XDScanWifiActivity
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.MainScope

@SuppressLint("MissingPermission")
class XDScanBLENormalFragment(private val host: XDScanBLEStatus.Host) : Fragment(),
    CoroutineScope by MainScope() {
    private val adapter = XDScanBLEAdapter {
        val intent = Intent(XDConnectService.ACTION_SELECT_BLE_DEVICE)
        intent.putExtra(XDConnectService.ACTION_SELECT_BLE_DEVICE_DATA, it)
        requireActivity().sendBroadcast(intent)
        XDScanWifiActivity.start(requireContext())
    }

    private lateinit var selectWifi: ActivityResultLauncher<Intent>

    private lateinit var bluetoothReceiver: BroadcastReceiver

    private lateinit var restartScanButton: Button

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_scanble_normal, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        selectWifi = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            it.data
        }

        restartScanButton = view.findViewById<Button>(R.id.scan_again)
        restartScanButton.setOnClickListener { restartScan() }

        val recyclerView = view.findViewById<RecyclerView>(R.id.scan_list)
        val layoutManager = GridLayoutManager(requireContext(), 2, RecyclerView.VERTICAL, false)

        recyclerView.layoutManager = layoutManager
        recyclerView.adapter = adapter
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        startService()
    }

    private fun startService() {
        bluetoothReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context?, intent: Intent) {
                when (intent.action) {
                    ACTION_START_SCAN -> {
                        restartScanButton.visibility = View.INVISIBLE
                    }

                    ACTION_STOP_SCAN -> {
                        restartScanButton.visibility = View.VISIBLE
                    }

                    ACTION_SCAN_RESULT -> {
                        val list = intent.getParcelableArrayListExtra(
                            ACTION_SCAN_RESULT_DATA, XDScanBLEDevice::class.java
                        )

                        if (list != null) {
                            setAdapterValue(list)
                        }
                    }
                }
            }
        }

        val intentFilter = IntentFilter()
        intentFilter.addAction(ACTION_START_SCAN)
        intentFilter.addAction(ACTION_STOP_SCAN)
        intentFilter.addAction(ACTION_SCAN_RESULT)
        ContextCompat.registerReceiver(
            requireContext(),
            bluetoothReceiver,
            intentFilter,
            ContextCompat.RECEIVER_EXPORTED,
        )

        ContextCompat.startForegroundService(
            requireContext(),
            Intent(requireContext(), XDConnectService::class.java),
        )
    }

    private fun stopService() {
        requireContext().stopService(
            Intent(
                requireContext(), XDConnectService::class.java
            )
        )
    }

    private fun restartScan() {
        setAdapterValue(listOf())
        val intent = Intent(XDConnectService.ACTION_RESTART_SCAN)
        requireActivity().sendBroadcast(intent)
    }

    private fun setAdapterValue(list: List<XDScanBLEDevice>) {
        adapter.setValues(list)
    }

    override fun onDetach() {
        stopService()
        super.onDetach()
    }
}